%% extracted from: Xuchun Li, Lei Wang, Eric Sung - "A Study of AdaBoost with SVM Based Weak Learners" - IJCNN 2005

function [Accuracy Specificity Sensitivity alpha weakres] = ClusterLVQ_AdaBoost_Intensities (learning_rate, learning_function)

   load /home/maite/Documentos/IWANN_IWINAC_IDEAL2009/codigo_IWANN09/spm_segmented_oasis_females.mat

   %Intensidades de voxel
   [C P] = extractVoxelIntensitiesFromEachCluster (female_crystal_brain_cov0, CONTROLES, PACIENTES);

   [L, num_clusters] = bwlabeln (female_crystal_brain_cov0);



   k_de_kfold = 10;
   p = cvpartition(size(P,1),'kfold',k_de_kfold);
   c = cvpartition(size(C,1),'kfold',k_de_kfold);
   control_label=2;
   patient_label=1;
  
   %%Parametros por defecto de las LVQ
   epochs     = 150;
   goal       = 0.01;
   show       = Inf;
   showWindow = false;
   hidden_neurons=5;
   %learning_rate=0.1;
   
   %classify and evaluate adaboost
   total_tests = 0;
   TP = 0;
   TN = 0;
   FP = 0;
   FN = 0;

   %weights for the classifier of each cluster, will be used for the final adaboost classifier
   alpha = ones(1,num_clusters)/num_clusters;

   %start training adaboost cluster SVM
   for k=1:k_de_kfold;

      %compute folds
      %training set used to compute the weights for each clusterSVM (our weaklearners)

      C_training = C(training (c,k) == 1,:,:);
      P_training = P(training (p,k) == 1,:,:);

      %test set used to validate the adaboost
      C_test     = C(training (c,k) == 0,:,:);
      P_test     = P(training (p,k) == 0,:,:);

      training_patients_no = size(P_training,1);
      training_controls_no = size(C_training,1);
      training_subjects_no = training_patients_no + training_controls_no;
      sample_weight = 1/training_subjects_no;

      %weight training
      for i=1:num_clusters;
          
%Para intensidades de voxel
         control_train_data = extractVoxelIntensitiesFromSubjectData (C_training,i);
         patient_train_data = extractVoxelIntensitiesFromSubjectData (P_training,i);
          
         %training
         [weight_training_data(i).data, weight_training_data(i).label, feat_no] = alignAndLabelDataForNN (control_train_data, patient_train_data, control_label, patient_label);
         label_ind=ind2vec(weight_training_data(i).label);
         
          %creating neural network and training with training set
          numTrain = training_subjects_no;
          priori_1 = training_controls_no/numTrain;
          priori_2 = training_patients_no/numTrain;

          lvqnet = newlvq(minmax(weight_training_data(i).data'), hidden_neurons, [priori_1 priori_2], learning_rate, learning_function);
          lvqnet.trainParam.epochs     = epochs;
          lvqnet.trainParam.goal       = goal;
          lvqnet.trainParam.show       = show;
          lvqnet.trainParam.showWindow = showWindow;

          [trained_net, train_record] = train(lvqnet, weight_training_data(i).data', label_ind);

         weaklearners(i).net = trained_net;

         %write training data to text
         %path=strcat('cluster',num2str(i),'_trainingdata_', num2str(k),'.txt');
         %writeDataToLIBSVMFormat (path, weight_training_data(i).data, weight_training_data(i).label);

         %classification test with training data

         results(k,i).output = vec2ind ( sim (weaklearners(i).net, weight_training_data(i).data') );
         results(k,i).target = weight_training_data(i).label;
         [weakres(k,i).TP weakres(k,i).TN weakres(k,i).FP weakres(k,i).FN weakres(k,i).Accuracy weakres(k,i).Specificity weakres(k,i).Sensitivity] = computeErrors (results(k,i).target', results(k,i).output);
         
         %adaboost compute training errors
         training_errors(k,i) = 0;
         for s=1:size(weight_training_data(i).data,1);
            if (results (k,i).output(s) ~= weight_training_data(i).label(s));
               training_errors(k,i) = training_errors(k,i) + sample_weight;
            end;
         end;

         %adaboost set classifier weights
         alpha (k,i) = 0.5 * log((1-training_errors(k,i)) / training_errors(k,i));
      end;

      %classify test set
      for i=1:num_clusters;
          
%Para intensidades de voxel          
         control_test_data = extractVoxelIntensitiesFromSubjectData (C_test,i);
         patient_test_data = extractVoxelIntensitiesFromSubjectData (P_test,i);

         
         [test_data(i).data, test_data(i).label] = alignAndLabelDataForNN (control_test_data, patient_test_data, control_label, patient_label);
         label_ind=ind2vec(weight_training_data(i).label);
         boosted_results(k,i).output = vec2ind ( sim (weaklearners(i).net, test_data(i).data') );
         boosted_results(k,i).target = test_data(i).label;
         
         %write test data to text
         %path=strcat('cluster',num2str(i),'_testdata_', num2str(k),'.txt');
         %writeDataToLIBSVMFormat (path, test_data(i).data, test_data(i).label);
      end;     
      
      %Transformamos los labels controles=1 y pacientes=-1 para
      %calcular la clasificacion el cjto de weaklearners
     trans_boosted_results=boosted_results;
     for j=1:num_clusters;

          trans_boosted_results(k,j).output(boosted_results(k,j).output==control_label) = 1;
          trans_boosted_results(k,j).output(boosted_results(k,j).output==patient_label) =-1;

          trans_boosted_results(k,j).target(boosted_results(k,j).target==control_label) = 1;
          trans_boosted_results(k,j).target(boosted_results(k,j).target==patient_label) =-1;
      
     end
      
      for s=1:size(test_data(1).data,1);
          
         %adaboost classification with alpha (weaklearner weights)
         %if negative: class = -1
         %else if positive: class = 1
         %else if zero: only the first cluster SVM prediction with maximum weight will be taken into account
         
         class = 0;
         for l=1:num_clusters;
            class = class + alpha(l) * trans_boosted_results (k,l).output(s);
         end;
         if (class == 0);
            [max_value max_idx] = max(alpha(:));
            class = trans_boosted_results (k,max_idx(1)).output(s);
         end;
         
         subj_predictions(s) = sign(class);

         %counting and compare with target label
         target_label = trans_boosted_results(k,l).target(s);

         if (target_label == -1); %positive        
            if   (subj_predictions(s) == -1); TP = TP + 1;
            else                              FN = FN + 1;
            end;
         else %negative
            if   (subj_predictions(s) ==  1); TN = TN + 1;
            else                              FP = FP + 1;
            end;
         end;

         total_tests = total_tests + 1;
         
         Partial_Accuracy (k) = (TP + TN)/total_tests;
         Partial_Specificity (k) = TN / (TN+FP);
         Partial_Sensitivity (k) = TP / (TP+FN);
                
      end;
      
   %   predictions(k,:) = subj_predictions(:);

      clear weaklearners subj_predictions
   end;

   Accuracy    = sum(Partial_Accuracy(:))/ k_de_kfold
   Specificity = sum(Partial_Specificity(:))/ k_de_kfold
   Sensitivity = sum(Partial_Sensitivity(:))/ k_de_kfold


