%% extracted from: Xuchun Li, Lei Wang, Eric Sung - "A Study of AdaBoost with SVM Based Weak Learners" - IJCNN 2005

function [Accuracy Specificity Sensitivity] = ClusterSVM_AdaBoost (WM_CONTROLS, WM_PATIENTS, SPM)

%   load ../Female/spm/spm_segmented_oasis_females.mat

   [C P] = extractMeanAndStdDevFromEachCluster (SPM, WM_CONTROLS, WM_PATIENTS);

   [L, num_clusters] = bwlabeln (SPM);

   libsvm_parameters = '-s 0 -t 0';%'-s 0 -t 0'; %'-s 0 -t 2 -g 0.031');%'-s 0 -t 0'); '

   k_de_kfold = 10;
   c = cvpartition(size(C,1),'kfold',k_de_kfold);
   p = cvpartition(size(P,1),'kfold',k_de_kfold);

   %weights for the classifier of each cluster, will be used for the final adaboost classifier
   alpha = ones(1,num_clusters)/num_clusters;

   %classify and evaluate adaboost
   total_tests = 0;
   TP = 0;
   TN = 0;
   FP = 0;
   FN = 0;

   %start training adaboost cluster SVM
   for k=1:k_de_kfold;

      %compute folds
      %training set used to compute the weights for each clusterSVM (our weaklearners)
      C_training = C(training (c,k) == 1,:,:);
      P_training = P(training (p,k) == 1,:,:);

      %test set used to validate the adaboost
      C_test     = C(training (c,k) == 0,:,:);
      P_test     = P(training (p,k) == 0,:,:);

      training_patients_no = size(P_training,1);
      training_controls_no = size(C_training,1);
      training_subjects_no = training_patients_no + training_controls_no;
      sample_weight = 1/training_subjects_no;

      %weight training
      for i=1:num_clusters;
         %training
         [weight_training_data(i).data, weight_training_data(i).label] = alignAndLabelData (C_training(:,i,:), P_training(:,i,:));

         %compute best gamma for this training set
         best_gamma(k,i) = computeBestRBFGamma (weight_training_data(i).data, weight_training_data(i).label);

         weaklearners(i) = svmtrain (weight_training_data(i).label, weight_training_data(i).data, sprintf ('-s 0 -t 2 -c 1 -g %f', best_gamma(k,i)));

         %write training data to text
         %path=strcat('cluster',num2str(i),'_trainingdata_', num2str(k),'.txt');
         %writeDataToLIBSVMFormat (path, weight_training_data(i).data, weight_training_data(i).label);

         %classification test with training data
         [results(k,i).predict_label, results(k,i).accuracy, results(k,i).prob_estimates] = svmpredict (weight_training_data(i).label, weight_training_data(i).data, weaklearners(i));

         %adaboost compute training errors
         training_errors(k,i) = 0;
         for s=1:size(weight_training_data(i).data,1);
            if (results (k,i).predict_label(s) ~= weight_training_data(i).label(s));
               training_errors(k,i) = training_errors(k,i) + sample_weight;
            end;
         end;

         %adaboost set classifier weights
         alpha (k,i) = 0.5 * log((1-training_errors(k,i)) / training_errors(k,i));
      end;

      %classify test set
      for i=1:num_clusters;
         [test_data(i).data, test_data(i).label] = alignAndLabelData (C_test(:,i ,:), P_test(:,i ,:));
         [boosted_results(k,i).predict_label, boosted_results(k,i).accuracy, boosted_results(k,i).prob_estimates] = svmpredict (test_data(i).label, test_data(i).data, weaklearners(i));

         %write test data to text
         %path=strcat('cluster',num2str(i),'_testdata_', num2str(k),'.txt');
         %writeDataToLIBSVMFormat (path, test_data(i).data, test_data(i).label);
      end;

      for s=1:size(test_data(1).data,1);

         %adaboost classification with alpha (weaklearner weights)
         %if negative: class = -1
         %else if positive: class = 1
         %else if zero: only the first cluster SVM prediction with maximum weight will be taken into account
         class = 0;
         for nc=1:num_clusters;
            class = class + alpha(nc) * boosted_results (k,nc).predict_label(s);
         end;
         if (class == 0);
            [max_value max_idx] = max(alpha);
            class = boosted_results (k,max_idx(1)).predict_label(s);
         end;

         subj_predictions(s) = sign(class);

         %counting and compare with target label
         target_label = test_data(1).label(s);

         if (test_data(1).label(s) == -1); %positive
            if   (subj_predictions(s) == -1); TP = TP + 1;
            else                              FN = FN + 1;
            end;
         else %negative
            if   (subj_predictions(s) ==  1); TN = TN + 1;
            else                              FP = FP + 1;
            end;
         end;

         total_tests = total_tests + 1;

      end;

   %   predictions(k,:) = subj_predictions(:);

      clear subj_predictions;

   end; %k-fold

   Accuracy    = (TP + TN) / total_tests
   Specificity = TN / (TN+FP)
   Sensitivity = TP / (TP+FN)


