%% reference: Xuchun Li, Lei Wang, Eric Sung - "A Study of AdaBoost with SVM Based Weak Learners" - IJCNN 2005

function [Accuracy Specificity Sensitivity weaklearners_info] = Diverse_AdaBoostSVM (glass_brain, control_data, patient_data)
   %cd /media/xcraft/Documents/phd/medical_images/my_papers/IWANN09/src

%   load ../Female/spm/spm_segmented_oasis_females.mat
%   [C P] = extractVoxelIntensitiesWithinClusters (female_crystal_brain_cov0, CONTROLES, PACIENTES);
%   [C P] = extractMeanAndStdDevFromEachCluster   (female_crystal_brain_cov0, CONTROLES, PACIENTES);

   C = control_data;
   P = patient_data;

   [L, num_clusters] = bwlabeln (glass_brain);

   k_de_kfold = 10;
   p = cvpartition(size(P,1),'kfold',k_de_kfold);
   c = cvpartition(size(C,1),'kfold',k_de_kfold);

   sigma_min     = 0.1;
   sigma_step    = 0.1;
   min_diversity = 0.5;

   for k=1:k_de_kfold;

      sigma            = 50;
      i                =  1;
      total_diversity  =  0;
   
      %compute folds
      %training set used to compute the weights for each clusterSVM (our weaklearners)
      C_training = C(training (c,k) == 1,:,:);
      P_training = P(training (p,k) == 1,:,:);
   
      %test set used to validate the adaboost
      C_test     = C(training (c,k) == 0,:,:);
      P_test     = P(training (p,k) == 0,:,:);

      training_patients_no = size(P_training,1);
      training_controls_no = size(C_training,1);
      training_subjects_no = training_patients_no + training_controls_no;
      sample_weights       = ones(1,training_subjects_no)'/training_subjects_no;

      %initialize training set
      [training_data training_label] = alignAndLabelData (C_training, P_training);

      %weight training
      while (sigma > sigma_min);
         gamma = 1/(2*sigma^2);
         weaklearner = svmtrain (training_label, training_data, sprintf ('-s 0 -t 2 -c 1 -g %f', gamma) );

         %classification test with training data
        [predict_label, accuracy, prob_estimates] = svmpredict (training_label, training_data, weaklearner);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         %ESTO TIENE UN ERROR.... predict_label no sirve para calcular la diversity...
         %    tienes que hallar el signo del sumatorio de todas las clasificaciones ya hechas para este sujeto más la clasificación del weaklearner actual.
         
         %adaboost compute training errors
         training_error = 0;
         diversity      = 0;
         for s=1:size(training_data,1);
            if (predict_label(s) ~= training_label(s));
               training_error = training_error  + sample_weights(s);
               diversity      = diversity       + 1;
            end;
         end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

         current_diversity = total_diversity + ( diversity/(i*training_subjects_no) )
         if ( (training_error > 0.50) || (diversity < min_diversity) ) %( current_diversity > min_diversity ) )
            sigma = sigma - sigma_step;
         else
            %save the weaklearner, save the world!
            weaklearners(i) = weaklearner;
         
            results(i).predict_label  = predict_label;
            results(i).accuracy       = accuracy;
            results(i).prob_estimates = prob_estimates;
            results(i).training_error = training_error;
            results(i).diversity      = diversity;
            results(i).sigma          = sigma;

            training_errors (i) = training_error;
            diversities     (i) = diversity;

            %compute total_diversity
            total_diversity = current_diversity;

            %adaboost set classifier weights
            alpha(i) = 1;
            if (training_errors(i) ~= 0);
               alpha (i) = 0.5 * log((1-training_errors(i)) / training_errors(i));
            end;
       
            results(i).weights = alpha(i);

            sample_weights = sample_weights .* exp( -alpha(i) * training_label .* results(i).predict_label);
            sample_weights = sample_weights/sum(sample_weights);

            %update sigma and loop counter for the next iteration
            sigma = sigma - sigma_step;
            i = i + 1;
         end;
      end;

      %classify test set
      [test_data, test_label] = alignAndLabelData (C_test, P_test);

      for i=1:size(alpha,1);
         [boosted_results(i).predict_label, boosted_results(i).accuracy, boosted_results(i).prob_estimates] = svmpredict (test_label, test_data, weaklearners(i));
      end;

      %classify and evaluate adaboost
      total_tests = 0;
      TP = 0;
      TN = 0;
      FP = 0;
      FN = 0;

      for s=1:size(test_data,1);

         %adaboost classification with alpha (weaklearner weights)
         %if negative: class = -1
         %else if positive: class = 1
         %else if zero: only the first cluster SVM prediction with maximum weight will be taken into account
         class = 0;
         for cl=1:size(alpha,1);
            class = class + alpha(cl) * boosted_results (cl).predict_label(s);
         end;
         if (class == 0);
            [max_value max_idx] = max(mean_alpha);
            class = boosted_results (max_idx(1)).predict_label(s);
         end;

         subj_predictions(s) = sign(class);

         %counting and compare with target label
         target_label = test_label(s);

         if (test_label(s) == -1); %positive
            if   (subj_predictions(s) == -1); TP = TP + 1;
            else                              FN = FN + 1;
            end;
         else %negative
            if   (subj_predictions(s) ==  1); TN = TN + 1;
            else                              FP = FP + 1;
            end;
         end;

         total_tests = total_tests + 1;
      end;

   end;

   %alpha

   Accuracy    = (TP + TN) / total_tests;
   Specificity = TN / (TN+FP);
   Sensitivity = TP / (TP+FN);
   weaklearners_info = results;

