function [Accuracy Specificity Sensitivity BestGamma BestC] = SVM_KFoldCrossValidation (C, P, control_label, patient_label, folds)

%load ../data/3-spm_and_wm_subjects_and_extracted_feats.mat
%C = meanstd_C;
%P = meanstd_P;

p = cvpartition(size(C,1), 'kfold', folds);
c = cvpartition(size(P,1), 'kfold', folds);
total_tests = 0;
TP = 0;
TN = 0;
FP = 0;
FN = 0;

for k=1:folds;

   C_training = C(training (c,k) == 1,:,:);
   P_training = P(training (p,k) == 1,:,:);
   C_test     = C(training (c,k) == 0,:,:);
   P_test     = P(training (p,k) == 0,:,:);
   total_results (k) = 0;

   [training_data(k).data, training_data(k).label] = alignAndLabelData (C_training, P_training, control_label, patient_label);
   [    test_data(k).data,     test_data(k).label] = alignAndLabelData (C_test,     P_test,     control_label, patient_label);

   %standardize data
   train_mean            = mean(mean(mean(training_data(k).data)));
   training_data(k).data = (training_data(k).data - train_mean) ./ 2;
       test_data(k).data = (    test_data(k).data - train_mean) ./ 2;

   [best_gamma best_C] = estimateBestRBFParameters (training_data(k).data, training_data(k).label);

   models(k) = svmtrain (training_data(k).label, training_data(k).data, sprintf ('-s 0 -t 2 -c %f -g %f', best_C, best_gamma));

   %classification
   [results(k).predict_label, results(k).accuracy, results(k).prob_estimates] = svmpredict (test_data(k).label, test_data(k).data, models(k));

   for s=1:size(test_data(1).data,1);

      class               = results (k).predict_label(s);
      subj_predictions(s) = sign(class);

      %counting and compare with target label
      target_label = test_data(1).label(s);

      if (test_data(1).label(s) == -1); %positive
         if   (subj_predictions(s) == -1); TP = TP + 1;
         else                              FN = FN + 1;
         end;
      else %negative
         if   (subj_predictions(s) ==  1); TN = TN + 1;
         else                              FP = FP + 1;
         end;
      end;

      total_tests = total_tests + 1;

   end;

%   predictions(k,:) = subj_predictions(:);

   clear subj_predictions;

end;
Accuracy    = (TP + TN) / total_tests
Specificity = TN / (TN+FP)
Sensitivity = TP / (TP+FN)
BestGamma   = best_gamma
BestC       = best_C


